/*
** Copyright (c) 1996, 2002
**      Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that: (1) Redistributions of
** source code must retain the above copyright notice, this list of
** conditions and the following disclaimer, (2) Redistributions in
** binary form must reproduce the above copyright notice, this list
** of conditions and the following disclaimer in the documentation
** and/or other materials provided with the distribution, (3) All
** advertising materials mentioning features or use of this software
** must display the following acknowledgment: ``This product includes
** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
** The name of the author may not be used to endorse or promote products
** derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

/*
 * $Id: NetintfP.h,v 1.12 2004/01/24 04:36:57 forys Exp $
 */

#ifndef	_XawNetintfP_h
#define	_XawNetintfP_h

#include "paths.h"
#include "Netintf.h"
#include <X11/Xaw/SimpleP.h>

#define	IfChkinc	1		/* inc "ival" after each intf check */
#define	IfChkmax	30		/* max secs before next intf check */
#define	IfChkmaxStr	"30"		/* must be same as "IfChkmax" */
#define	IfDefault	"ppp0"		/* default interface */
#define	VolumeStr	"33"		/* default bell volume (percentage) */
#define NoifStr		"0"		/* default: interface must exist */
#define	DbglvlStr	"0"		/* default debug level (no debug) */

typedef enum {
	IFST_DOWN,			/* interface is down */
	IFST_TOINIT,			/* in transition to IFST_INIT */
	IFST_INIT,			/* interface being initialized */
	IFST_RUN,			/* interface is running */
	IFST_UP,			/* interface is up */
	IFST_TODOWN,			/* in transition to IFST_DOWN */
	IFST_ERR			/* cant determine interface state */
} NetintfState;
#define	SETSTATE(w, ifstate)		w->netintf.state = ifstate
#define	GETSTATE(w)			w->netintf.state

typedef struct {
	/* resources */
	String ifname;			/* name of interface to monitor */
	String confdir;			/* configuration directory */
	int maxival;			/* max secs to wait before intf chk */
	int volume;			/* bell volume */
	int noif;			/* 0 when interface must exist */
	Pixel fg_pixel;			/* foreground color index */
	XFontStruct *font;		/* font for "ifname" display */
	int dbglvl;			/* debug level */
	/* local state */
	GC gc;				/* GC for interface */
	NetintfState state;		/* current interface state */
	int sockfd;			/* interface file descriptor */
	int wid, hgt;			/* size of "ifname" in "font" */
	int ival;			/* secs to next interface check */
	XtIntervalId intid;		/* id of interval timer (0 if none) */
	struct _ifimage {		/* list of interface bitmaps */
		Pixmap bitmap;
		Pixmap pixmap;
		int width, height;
	} down, tran, init, run, up, err;
} NetintfPart;

typedef struct _NetintfRec {
	CorePart core;
	SimplePart simple;
	NetintfPart netintf;
} NetintfRec;

typedef struct { int dummy; } NetintfClassPart;

typedef struct _NetintfClassRec {
	CoreClassPart core_class;
	SimpleClassPart simple_class;
	NetintfClassPart netintf_class;
} NetintfClassRec;

extern NetintfClassRec netintfClassRec;

/*
 * Network Interface Operation prototypes.
 */
void		NetIfDn _P_((NetintfWidget));
void		NetIfUp _P_((NetintfWidget));
Boolean		NetIfIsInit _P_((NetintfWidget));
void		NetIfExecUp _P_((NetintfWidget));
void		NetIfExecDn _P_((NetintfWidget));
NetintfState	NetIfState _P_((NetintfWidget));

#endif	/* _XawNetintfP_h */
