/*
** Copyright (c) 1996, 2002
**      Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that: (1) Redistributions of
** source code must retain the above copyright notice, this list of
** conditions and the following disclaimer, (2) Redistributions in
** binary form must reproduce the above copyright notice, this list
** of conditions and the following disclaimer in the documentation
** and/or other materials provided with the distribution, (3) All
** advertising materials mentioning features or use of this software
** must display the following acknowledgment: ``This product includes
** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
** The name of the author may not be used to endorse or promote products
** derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

/*
** ifsuex - verify user and execute interface command.
*/

const char *Version = "1.3 20020506";

const char *CopyrightVersion = "ifsuex %s\n\nCopyright (c) 1996, 2002 Jeff Forys.  All rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, is permitted under the terms specified in the source\ncode (and manual pages).\n";

#ifndef lint
static char rcsid[] = "$Id: ifsuex.c,v 1.14 2002/05/04 02:09:15 forys Exp $";
#endif

/*
 * Things you may wish to change...
 */

#define	HAVE_STRERROR		/* have strerror() */
#define	HAVE_SNPRINTF		/* have snprintf() ("safe" without it) */
#define	HAVE_STRCASECMP		/* have strcasecmp() (o/w case significant) */
#define	USE_SYSLOG		/* log errors to syslog (o/w only stderr) */
#undef	USE_SETREUID		/* Use setreuid() rather than setuid() */

#define	MSGBUFSIZ	256	/* error buffer size */
#define	CFLINELEN	1024	/* maximum line length for config file */
#define	EXARGMAX	256	/* maximum number of arguments to execve() */

/*
 * End of configurables.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <errno.h>

#include "paths.h"

#ifdef	__alpha			/* some vendors (e.g., Sun) define BSD empty */
#include <sys/param.h>
#if defined(BSD) && (BSD == 198911)	/* assume Tru64 */
#define USE_SETREUID
#endif
#endif	/* __alpha */

#if defined(__STDC__) || defined(__cplusplus)
#define	_P_(protos)	protos
#else
#define	_P_(protos)	()
#define	const
#endif

#ifdef	USE_SYSLOG
#include <syslog.h>
#else
#ifndef	LOG_ERR
#define	LOG_ERR		3	/* meaningless: source code compat only */
#endif
#ifndef	LOG_INFO
#define	LOG_INFO	6	/* meaningless: source code compat only */
#endif
#endif

#ifdef	HAVE_STRCASECMP
#define	STREQU(s1, s2)		(strcasecmp(s1, s2) == 0)
#else
#define	STREQU(s1, s2)		(strcmp(s1, s2) == 0)
#endif

#ifdef	HAVE_SNPRINTF
#define	SPRINTF2(buf,a1,a2)	snprintf(buf,MSGBUFSIZ,"%s: %s", a1, a2)
#define	SPRINTF2D(buf,a1,a2)	snprintf(buf,MSGBUFSIZ,"%s: %d", a1, a2)
#define	SPRINTF3(buf,a1,a2,a3)	snprintf(buf,MSGBUFSIZ,"%s: %s: %s", a1, a2, a3)
#else
#define	SPRINTF2(buf,a1,a2)	sprintf(buf,"%s: %s", a1, a2)
#define	SPRINTF2D(buf,a1,a2)	sprintf(buf,"%s: %d", a1, a2)
#define	SPRINTF3(buf,a1,a2,a3)	sprintf(buf,"%s: %s: %s", a1, a2, a3)
#endif

#ifdef	HAVE_STRERROR
#define	STRERROR		strerror
#else
#define	STRERROR		my_strerror
const char *			my_strerror _P_((int));
#endif

#ifndef TRUE
#define	TRUE	1
#endif
#ifndef FALSE
#define	FALSE	0
#endif

#define	CF_UP	0		/* config file: "up" keyword */
#define	CF_DOWN	1		/* config file: "down" keyword */
#define	CF_USER	2		/* config file: "user" keyword */

typedef struct {
	char *ifup[EXARGMAX];	/* argv for bringing interface up */
	char *ifdn[EXARGMAX];	/* argv for bringing interface down */
	char lineup[CFLINELEN];	/* strings in *ifup[] argument vector */
	char linedn[CFLINELEN];	/* strings in *ifdn[] argument vector */
} itab_t;

int	verbose = 0;		/* verbose operation */
int	noexec = 0;		/* do not exec command (error check mode) */
int	intfup = 0;		/* set if "-u" specified */
int	intfdn = 0;		/* set if "-d" specified */
char	*intfname = NULL;	/* set to <intf> in "-i <intf>" */
char	*ProgName;		/* path-stripped "argv[0]" */
char	userenv[128+6];		/* validated "USER=" line to child env */

/*
 * Prototypes.
 */
itab_t *getintf _P_((const char *, const char *, char *));
char *	gettok _P_((char **));
void	errexit _P_((int, char *, int));
char **	newenv _P_((char **));
void	usage _P_(());

int
main(argc, argv)
	int argc;
	char *argv[];
{
	extern char **environ;
	extern char *optarg;
	extern int optind;
	struct passwd *pw;
	itab_t *itab;
	char msgbuf[MSGBUFSIZ], **avec;
	uid_t myuid;
	int c;

	/*
	 * Get path-stripped program name and invoking user's name.
	 */
	if ((ProgName = strrchr(*argv, '/')) != NULL)
		ProgName++;
	else
		ProgName = *argv;

#ifdef USE_SYSLOG
	openlog(ProgName, LOG_CONS, LOG_USER);
#endif

	if ((pw = getpwuid(myuid = getuid())) == NULL) {
		SPRINTF2D(msgbuf, "can not determine user name", (int)myuid);
		errexit(LOG_ERR, msgbuf, 2);
	}
#ifdef	HAVE_SNPRINTF
	snprintf(userenv, sizeof userenv, "USER=%s", pw->pw_name);
#else
	sprintf(userenv, "USER=%s", pw->pw_name);
#endif

	/*
	 * Parse arguments, call usage()/exit() if anything is wrong.
	 */
	while ((c = getopt(argc, argv, "vnud")) != EOF) {
		switch(c) {
		    case 'v':
			if (verbose)
				usage();
			verbose = 1;
			break;
		    case 'n':
			if (noexec)
				usage();
			noexec = 1;
			break;
		    case 'u':
			if (intfup || intfdn)
				usage();
			intfup = 1;
			break;
		    case 'd':
			if (intfup || intfdn)
				usage();
			intfdn = 1;
			break;
		    case '?':
		    default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;

	if (argc > 0) {
		intfname = *argv++;
		argc--;
	}

	/* "-v" alone prints version information and exits successfully */
	if (verbose && !noexec && !intfup && intfname == NULL) {
		printf(CopyrightVersion, Version);
		return 0;
	}

	/*
	 * Ensure that we got one of "-u" or "-d", that we have an
	 * interface name, and that there are no unexpected parameters.
	 */
	if ((intfup == 0 && intfdn == 0) || intfname == NULL || argc > 0)
		usage();

	/*
	 * Get calling sequence for interface "intfname".  If getintf()
	 * returns NULL, "errmsg" contains the error message.
	 */
	if ((itab = getintf(intfname, pw->pw_name, msgbuf)) == NULL)
		errexit(LOG_ERR, msgbuf, 1);

	/*
	 * Set argument vector, remove dangerous stuff from environment,
	 * and, if *avec is non-NULL, exec program.
	 */
	avec = intfup? itab->ifup: itab->ifdn;
	environ = newenv(environ);

#ifdef  USE_SETREUID
	setreuid(geteuid(), -1);
#else
	setuid(geteuid());
#endif

	if (verbose) {
		char **avp;

		printf("%s: exec cmd:", ProgName);
		for (avp = avec; *avp != NULL; avp++)
			printf(" %s", *avp);
		printf("\n");
		fflush(stdout);
	}
#ifdef SYSLOG
	SPRINTF3(msgbuf, intfname, pw->pw_name,
		intfup? "bringing interface up": "bringing interface down");
	syslog(LOG_INFO, msgbuf);
#endif

	if (*avec != NULL) {
		execve(*avec, (char * const *)avec, environ);

		/*
		 * execve() failed; display error message and exit().
		 */
		SPRINTF3(msgbuf, "execve", *avec, STRERROR(errno));
		errexit(LOG_ERR, msgbuf, 2);
	}

	return 0;
}

/*
 * getintf(ifname, user, errmsg)
 *
 * Search through configuration file for interface "ifname" with permission
 * for "user" to execute command.  If both conditions are satisfied, return
 * pointer to itab entry.  Otherwise return NULL and set "errmsg" to an
 * error message.
 */
itab_t *
getintf(ifname, user, errmsg)
	const char *ifname, *user;
	char *errmsg;
{
	static char cfline[CFLINELEN];
	static itab_t intf;
	FILE *fp;
	char *tok, *cp, *cferrmsg;
	int i, lineno, luline;
	int haveintf = 0;
	int haveuser = 0;
	int haveup = 0;
	int havedown = 0;

	if (chdir(_PATH_XNETINTF) < 0) {
		SPRINTF2(errmsg, _PATH_XNETINTF, STRERROR(errno));
		return NULL;
	}

	if ((fp = fopen(_PATH_IFSUEXCF, "r")) == NULL) {
		SPRINTF3(errmsg, _PATH_XNETINTF, _PATH_IFSUEXCF,
		         "can not open config file");
		return NULL;
        }

	lineno = luline = 0;
	while (fgets(cfline, CFLINELEN, fp) != NULL) {
		lineno++;
		if (*cfline == '#')
			continue;
		cp = cfline;

		/*
		 * Get configuration command.
		 */
		if ((tok = gettok(&cp)) == NULL)
			continue;
		if (STREQU(tok, "Up") != 0)
			i = CF_UP;
		else if (STREQU(tok, "Down") != 0)
			i = CF_DOWN;
		else if (STREQU(tok, "User") != 0)
			i = CF_USER;
		else {
			SPRINTF3(errmsg, _PATH_IFSUEXCF, tok, "invalid token");
			return NULL;
		}

		/*
		 * Get interface name.
		 * If not the one we are looking for, go to next line.
		 */
		cferrmsg = "unexpected end of line";
		if ((tok = gettok(&cp)) == NULL)
			goto cferror;
		if (!STREQU(tok, ifname))
			continue;

		haveintf = 1;

		switch (i) {
		    case CF_UP:
			if (haveup++ > 0) {
				cferrmsg ="multiple UP for interface";
				goto cferror;
			}
			for (i = 0, cp = strcpy(intf.lineup, cp);
			     (tok = gettok(&cp)) != NULL && i < EXARGMAX; i++)
				intf.ifup[i] = tok;
			intf.ifup[i] = NULL;
			break;
		    case CF_DOWN:
			if (havedown++ > 0) {
				cferrmsg ="multiple DOWN for interface";
				goto cferror;
			}
			for (i = 0, cp = strcpy(intf.linedn, cp);
			     (tok = gettok(&cp)) != NULL && i < EXARGMAX; i++)
				intf.ifdn[i] = tok;
			intf.ifdn[i] = NULL;
			break;
		    case CF_USER:
			luline = lineno;
			while (!haveuser && (tok = gettok(&cp)) != NULL)
				if (STREQU(tok, user))
					haveuser = 1;
			break;
		}
	}

	(void) fclose(fp);

	if (!haveintf) {
		cferrmsg = "no such interface";
		goto cferror;
	} else if (!haveuser) {
		cferrmsg = "permission denied";
		lineno = luline;
		goto cferror;
	} else if (intf.ifup == NULL) {
		cferrmsg = "no UP for interface";
		goto cferror;
	} else if (intf.ifdn == NULL) {
		cferrmsg = "no DOWN for interface";
		goto cferror;
	}

	return &intf;

cferror:
#ifdef HAVE_SNPRINTF
	snprintf(errmsg, MSGBUFSIZ, 
#else
	sprintf(errmsg,
#endif
		"%s: error on line %d: %s (intf %s) (user %s)",
		_PATH_IFSUEXCF, lineno, cferrmsg, ifname, user);
	return NULL;
}

/*
 * gettok(scp)
 *
 * Return next token; begin search at *scp, return pointer to start of next
 * token (null-terminated), and advance scp past it.  If no tokens remain,
 * return NULL.
 *
 * Note:  Any sequence of isspace() is a separator.
 */
char *
gettok(scp)
	char **scp;
{
	char *cp = *scp;
	char *bcp = NULL;

	while (*cp && isspace(*cp))	/* skip leading whitespace */
		cp++;
	if (*cp)			/* save start of token */
		bcp = cp;
	while (*cp && !isspace(*cp))	/* skip past token */
		cp++;
	if (*cp)			/* null-terminate and advance past */
		*cp++ = '\0';

	*scp = cp;

	return bcp;
}

/*
 * newenv()
 *
 * Return an array of pointers to selected environment variables
 * taken from 'environ'.  Permitted variables include:
 *
 *    HOME, LOGNAME, PATH, TZ, USER
 *
 * USER from the environment is actually ignored; it is always set to
 * the invokers real user name here so that callee knows the caller.
 */
char **
newenv(environ)
	char **environ;
{
	static char *env[6];
	register char **cpp, **cpp2;

	for (cpp = environ, cpp2 = env; *cpp; cpp++) {
		if (strncmp(*cpp, "HOME=", 5) == 0 ||
		    strncmp(*cpp, "LOGNAME=", 8) == 0 ||
		    strncmp(*cpp, "PATH=", 5) == 0 ||
		    strncmp(*cpp, "TZ=", 3) == 0)
			*cpp2++ = *cpp;
	}
	*cpp2++ = userenv;
	*cpp2 = 0;

	return env;
}

/*
 * usage()
 *
 * Display usage information and exit.
 */
void
usage()
{
	fprintf(stderr, "usage: %s [-vn] -(u|d) <intf>\n", ProgName);
	exit(1);
}

void
errexit(lvl, msg, excode)
	int lvl;
	char *msg;
	int excode;
{
#ifdef USE_SYSLOG
	syslog(lvl, msg);
#endif
	fprintf(stderr, "%s: %s\n", ProgName, msg);
	exit(excode);
}

#ifndef HAVE_STRERROR
const char *
my_strerror(num)
	int num;
{
	extern int sys_nerr;
	extern char *sys_errlist[];
	static char errnobuf[MSGBUFSIZ];
	register unsigned int errnum;

	errnum = num;				/* make it unsigned */
	if (errnum < sys_nerr)
		return sys_errlist[errnum];

	SPRINTF2D(errnobuf, "Unknown error", num);
	return errnobuf;
}
#endif
